# -*- coding: utf-8 -*-
"""Unit tests for account endpoints"""

import pytest
from requests.exceptions import HTTPError
from src.rev_ai.apiclient import RevAiAPIClient
from src.rev_ai.models import Account
from tests.helpers.errors import get_error_test_cases

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

URL = urljoin(RevAiAPIClient.base_url, 'account')


@pytest.mark.usefixtures('mock_client', 'make_mock_response')
class TestAccountEndpoints():
    def test_get_account_with_success(
            self, mock_client, make_mock_response):
        email = 'text@example.com'
        seconds = 10
        data = {'email': email, 'balance_seconds': seconds}
        response = make_mock_response(url=URL, json_data=data)
        mock_client.session.get.return_value = response

        res = mock_client.get_account()

        assert res == Account(email, seconds)
        mock_client.session.get.assert_called_once_with(URL)

    @pytest.mark.parametrize('error', get_error_test_cases(
        ['unauthorized']))
    def test_get_account_with_error_response(self, error, mock_client, make_mock_response):
        status = error.get('status')
        response = make_mock_response(url=URL, status=status, json_data=error)
        mock_client.session.get.return_value = response

        with pytest.raises(HTTPError, match=str(status)):
            mock_client.get_account()
        mock_client.session.get.assert_called_once_with(URL)



# -*- coding: utf-8 -*-
"""Mock RevAiAPIClient for testing purposes"""

import pytest
from requests import Response
from src.rev_ai.apiclient import RevAiAPIClient


@pytest.fixture
def mock_client(mocker):
    client = RevAiAPIClient('token')
    client.session.get = mocker.Mock(name='mock_get')
    client.session.post = mocker.Mock(name='mock_post')
    client.session.delete = mocker.Mock(name='mock_delete')
    return client


@pytest.fixture
def make_mock_response(mocker):
    def _mock_response(url="", status=200, json_data=None, text=""):
        response = Response()
        response.status_code = status
        response.reason = 'Testing'
        response.url = url
        if text:
            type(response).text = mocker.PropertyMock(return_value=text)
        if json_data:
            response.json = mocker.Mock(return_value=json_data)
        return response
    return _mock_response



# -*- coding: utf-8 -*-
"""Speech recognition tools for using Rev.ai"""

import requests
import json
from .models import Job, Account, Transcript
from . import __version__

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin


class RevAiAPIClient:
    """Client which implements Rev.ai API

    Note that HTTPErrors can be thrown by methods of the API client. The HTTP response payload
    attached to these error is a problem details. The problem details information is represented
    as a JSON object with error specific properties that help to troubleshoot the problem.

    Problem details are defined at https://tools.ietf.org/html/rfc7807.
    """

    # Default version of Rev.ai
    version = 'v1'

    # Default address of the API
    base_url = 'https://api.rev.ai/speechtotext/{}/'.format(version)

    # Rev.ai transcript format
    rev_json_content_type = 'application/vnd.rev.transcript.v1.0+json'

    #rev.ai transcript format
    rev_json_content_type = 'application/vnd.rev.transcript.v1.0+json'

    def __init__(self, access_token):
        """Constructor

        :param access_token: access token which authorizes all requests and links them to your
                             account. Generated on the settings page of your account dashboard
                             on Rev.ai.
        """
        if not access_token:
            raise ValueError('access_token must be provided')

        self.session = requests.Session()
        self.session.headers.update({
            'Authorization': 'Bearer {}'.format(access_token),
            'User-Agent': 'RevAi-PythonSDK/{}'.format(__version__)
        })

    def submit_job_url(
            self, media_url,
            metadata=None,
            callback_url=None,
            skip_diarization=False):
        """Submit media given a URL for transcription.
        The audio data is downloaded from the URL.

        :param media_url: web location of the media file
        :param metadata: info to associate with the transcription job
        :param callback_url: callback url to invoke on job completion as a webhook
        :param skip_diarization: should rev.ai skip diaization when transcribing this file
        :returns: raw response data
        :raises: HTTPError
        """
        if not media_url:
            raise ValueError('media_url must be provided')

        url = urljoin(self.base_url, 'jobs')
        payload = {'media_url': media_url}
        if metadata:
            payload['metadata'] = metadata
        if callback_url:
            payload['callback_url'] = callback_url

        response = self.session.post(url, json=payload)
        response.raise_for_status()

        return Job.from_json(response.json())

    def submit_job_local_file(
            self, filename,
            metadata=None,
            callback_url=None,
            skip_diarization=False):
        """Submit a local file for transcription.
        Note that the content type is inferred if not provided.

        :param filename: path to a local file on disk
        :param metadata: info to associate with the transcription job
        :param callback_url: callback url to invoke on job completion as a webhook
        :param skip_diarization: should rev.ai skip diaization when transcribing this file
        :returns: raw response data
        :raises: HTTPError
        """
        if not filename:
            raise ValueError('filename must be provided')

        url = urljoin(self.base_url, 'jobs')
        payload = {}
        if metadata:
            payload['metadata'] = metadata
        if callback_url:
            payload['callback_url'] = callback_url

        with open(filename, 'rb') as f:
            files = {
                'media': (filename, f),
                'options': (None, json.dumps(payload))
            }

            response = self.session.post(url, files=files)
            response.raise_for_status()

        return Job.from_json(response.json())

    def get_job_details(self, id_):
        """View information about a specific job.
        The server will respond with the status and creation date.

        :param id_: id of the job to be requested
        :returns: raw response data
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        url = urljoin(self.base_url, 'jobs/{}'.format(id_))
        response = self.session.get(url)
        response.raise_for_status()

        return Job.from_json(response.json())

    def get_list_of_jobs(self, limit=None, starting_after=None):
        """Get a list of transcription jobs submitted within the last week in reverse
        chronological order up to the provided limit number of jobs per call.
        Pagination is supported via passing the last job id from previous call into starting_after.

        :param limit: optional, limits the number of jobs returned,
                      if none, a default of 100 jobs is returned, max limit if 1000
        :param starting_after: optional, returns jobs created after the job with this id,
                               exclusive (job with this id is not included)
        :returns: list of jobs response data
        :raises: HTTPError
        """
        params = []
        if limit is not None:
            params.append('limit={}'.format(limit))
        if starting_after is not None:
            params.append('starting_after={}'.format(starting_after))

        query = '?{}'.format('&'.join(params))
        url = urljoin(self.base_url, 'jobs{}'.format(query))
        response = self.session.get(url)
        response.raise_for_status()

        return [Job.from_json(job) for job in response.json()]

    def get_transcript_text(self, id_):
        """Get the transcript of a specific job as plain text.

        :param id_: id of job to be requested
        :returns: transcript data as text
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        url = urljoin(self.base_url, 'jobs/{}/transcript'.format(id_))
        response = self.session.get(url, headers={'Accept': 'text/plain'})
        response.raise_for_status()

        return response.text

    def get_transcript_json(self, id_):
        """Get the transcript of a specific job as json

        :param id_: id of job to be requested
        :returns: transcript data as json
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        url = urljoin(self.base_url, 'jobs/{}/transcript'.format(id_))
        response = self.session.get(
            url, headers={'Accept': self.rev_json_content_type})
        response.raise_for_status()

        return response.json()

    def get_transcript_object(self, id_):
        """Get the transcript of a specific job as a python object`.

        :param id_: id of job to be requested
        :returns: transcript data as a python object
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        url = urljoin(self.base_url, 'jobs/{}/transcript'.format(id_))
        response = self.session.get(
            url, headers={'Accept': self.rev_json_content_type})
        response.raise_for_status()

        return Transcript.from_json(response.json())

    def delete_job(self, id_):
        """Delete a specific transcription job
        All data related to the job, such as input media and transcript, will be permanently
        deleted. A job can only by deleted once it's completed.

        :param id_: id of job to be deleted
        :returns: None if job was successfully deleted
        :raises: HTTPError
        """
        if not id_:
            raise ValueError('id_ must be provided')

        url = urljoin(self.base_url, 'jobs/{}'.format(id_))
        response = self.session.delete(url)
        response.raise_for_status()

        return

    def get_account(self):
        """Get account information, such as remaining balance.

        :raises: HTTPError
        """
        url = urljoin(self.base_url, 'account')
        response = self.session.get(url)
        response.raise_for_status()

        return Account.from_json(response.json())



# -*- coding: utf-8 -*-
"""Unit tests for transcript endpoints"""

import pytest
import json
from requests.exceptions import HTTPError
from src.rev_ai.models import Transcript, Monologue, Element
from src.rev_ai.apiclient import RevAiAPIClient
from tests.helpers.errors import get_error_test_cases

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

JOB_ID = '1'
URL = urljoin(RevAiAPIClient.base_url, 'jobs/{}/transcript'.format(JOB_ID))


@pytest.mark.usefixtures('mock_client', 'make_mock_response')
class TestTranscriptEndpoints():
    def test_get_transcript_text_with_success(self, mock_client, make_mock_response):
        data = 'Test'
        response = make_mock_response(url=URL, text=data)
        mock_client.session.get.return_value = response

        res = mock_client.get_transcript_text(JOB_ID)

        assert res == data
        mock_client.session.get.assert_called_once_with(URL, headers={'Accept': 'text/plain'})

    @pytest.mark.parametrize('id', [None, ''])
    def test_get_transcript_text_with_no_job_id(self, id, mock_client):
        with pytest.raises(ValueError, match='id_ must be provided'):
            mock_client.get_transcript_text(id)

    @pytest.mark.parametrize('error', get_error_test_cases(
        ['unauthorized', 'job-not-found', 'invalid-job-state']))
    def test_get_transcript_text_with_error_response(self, error, mock_client, make_mock_response):
        status = error.get('status')
        response = make_mock_response(url=URL, status=status, json_data=error)
        mock_client.session.get.return_value = response

        with pytest.raises(HTTPError, match=str(status)):
            mock_client.get_transcript_text(JOB_ID)
        mock_client.session.get.assert_called_once_with(URL, headers={'Accept': 'text/plain'})

    def test_get_transcript_json_with_success(self, mock_client, make_mock_response):
        data = {
            'monologues': [{
                'speaker': 1,
                'elements': [{
                    'type': 'text',
                    'value': 'Hello',
                    'ts': 0.75,
                    'end_ts': 1.25,
                    'confidence': 0.85
                }]
            }]
        }
        expected = json.loads(json.dumps(data))
        response = make_mock_response(url=URL, json_data=data)
        mock_client.session.get.return_value = response

        res = mock_client.get_transcript_json(JOB_ID)

        assert res == expected
        mock_client.session.get.assert_called_once_with(
            URL, headers={'Accept': 'application/vnd.rev.transcript.v1.0+json'})

    @pytest.mark.parametrize('id', [None, ''])
    def test_get_transcript_json_with_no_job_id(self, id, mock_client):
        with pytest.raises(ValueError, match='id_ must be provided'):
            mock_client.get_transcript_json(id)

    @pytest.mark.parametrize('error', get_error_test_cases(
        ['unauthorized', 'job-not-found', 'invalid-job-state']))
    def test_get_transcript_json_with_error_response(self, error, mock_client, make_mock_response):
        status = error.get('status')
        response = make_mock_response(url=URL, status=status, json_data=error)
        mock_client.session.get.return_value = response

        with pytest.raises(HTTPError, match=str(status)):
            mock_client.get_transcript_json(JOB_ID)
        mock_client.session.get.assert_called_once_with(
            URL, headers={'Accept': 'application/vnd.rev.transcript.v1.0+json'})

    def test_get_transcript_object_with_success(self, mock_client, make_mock_response):
        data = {
            'monologues': [{
                'speaker': 1,
                'elements': [{
                    'type': 'text',
                    'value': 'Hello',
                    'ts': 0.75,
                    'end_ts': 1.25,
                    'confidence': 0.85
                }]
            }]
        }
        expected = Transcript([Monologue(1, [Element('text', 'Hello', 0.75, 1.25, 0.85)])])
        response = make_mock_response(url=URL, json_data=data)
        mock_client.session.get.return_value = response

        res = mock_client.get_transcript_object(JOB_ID)

        assert res == expected
        mock_client.session.get.assert_called_once_with(
            URL, headers={'Accept': 'application/vnd.rev.transcript.v1.0+json'})

    @pytest.mark.parametrize('id', [None, ''])
    def test_get_transcript_object_with_no_job_id(self, id, mock_client):
        with pytest.raises(ValueError, match='id_ must be provided'):
            mock_client.get_transcript_object(id)

    @pytest.mark.parametrize('error', get_error_test_cases(
        ['unauthorized', 'job-not-found', 'invalid-job-state']))
    def test_get_transcript_object_with_error_response(self, error, mock_client, make_mock_response):
        status = error.get('status')
        response = make_mock_response(url=URL, status=status, json_data=error)
        mock_client.session.get.return_value = response

        with pytest.raises(HTTPError, match=str(status)):
            mock_client.get_transcript_object(JOB_ID)
        mock_client.session.get.assert_called_once_with(
            URL, headers={'Accept': 'application/vnd.rev.transcript.v1.0+json'})



# -*- coding: utf-8 -*-
"""Unit tests for job endpoints"""

import json
import pytest
from requests.exceptions import HTTPError
from src.rev_ai.apiclient import RevAiAPIClient
from src.rev_ai.models import Job, JobStatus
from tests.helpers.errors import get_error_test_cases

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

JOB_ID = '1'
METADATA = 'test'
CALLBACK_URL = 'https://callback.com/'
CREATED_ON = '2018-05-05T23:23:22.29Z'
MEDIA_URL = 'https://example.com/test.mp3'
FILENAME = 'test.mp3'
JOB_ID_URL = urljoin(RevAiAPIClient.base_url, 'jobs/{}'.format(JOB_ID))
JOBS_URL = urljoin(RevAiAPIClient.base_url, 'jobs')


@pytest.mark.usefixtures('mock_client', 'make_mock_response')
class TestJobEndpoints():
    def test_get_job_details_with_success(self, mock_client, make_mock_response):
        status = 'transcribed'
        created_on = '2018-05-05T23:23:22.29Z'
        data = {
            'id': JOB_ID,
            'status': status,
            'created_on': created_on
        }
        response = make_mock_response(url=JOB_ID_URL, json_data=data)
        mock_client.session.get.return_value = response

        res = mock_client.get_job_details(JOB_ID)

        assert res == Job(JOB_ID, created_on, JobStatus.TRANSCRIBED)
        mock_client.session.get.assert_called_once_with(JOB_ID_URL)

    @pytest.mark.parametrize('id', [None, ''])
    def test_get_job_details_with_no_job_id(self, id, mock_client):
        with pytest.raises(ValueError, match='id_ must be provided'):
            mock_client.get_job_details(id)

    @pytest.mark.parametrize('error', get_error_test_cases(
        ['unauthorized', 'job-not-found']))
    def test_get_job_details_with_error_response(self, error, mock_client, make_mock_response):
        status = error.get('status')
        response = make_mock_response(url=JOB_ID_URL, status=status, json_data=error)
        mock_client.session.get.return_value = response

        with pytest.raises(HTTPError, match=str(status)):
            mock_client.get_job_details(JOB_ID)
        mock_client.session.get.assert_called_once_with(JOB_ID_URL)

    def test_get_list_of_jobs_limit_with_success(self, mock_client, make_mock_response):
        status = 'transcribed'
        created_on = '2018-05-05T23:23:22.29Z'
        data = [
            {
                'id': JOB_ID,
                'status': status,
                'created_on': created_on
            },
            {
                'id': '2',
                'status': 'in_progress',
                'created_on': created_on
            }
        ]
        url = JOBS_URL + "?limit=2"
        response = make_mock_response(url=url, json_data=data)
        mock_client.session.get.return_value = response

        res = mock_client.get_list_of_jobs(limit=2)

        assert isinstance(res, list)
        assert len(res) == 2
        mock_client.session.get.assert_called_once_with(url)

    def test_get_list_of_jobs_starting_after_with_success(self, mock_client, make_mock_response):
        status = 'transcribed'
        created_on = '2018-05-05T23:23:22.29Z'
        data = [
            {
                'id': JOB_ID,
                'status': status,
                'created_on': created_on
            }
        ]
        url = JOBS_URL + "?starting_after=4"
        response = make_mock_response(url=url, json_data=data)
        mock_client.session.get.return_value = response

        res = mock_client.get_list_of_jobs(starting_after="4")

        assert isinstance(res, list)
        assert len(res) == 1
        mock_client.session.get.assert_called_once_with(url)

    @pytest.mark.parametrize('error', get_error_test_cases(
        ['invalid-parameters', 'unauthorized']))
    def test_get_list_of_jobs_with_error_response(self, error, mock_client, make_mock_response):
        status = error.get('status')
        response = make_mock_response(url=JOBS_URL, status=status, json_data=error)
        mock_client.session.get.return_value = response
        with pytest.raises(HTTPError, match=str(status)):
            mock_client.get_list_of_jobs()
        mock_client.session.get.assert_called_once_with(JOBS_URL)

    def test_submit_job_url_with_success(self, mock_client, make_mock_response):
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': CREATED_ON,
            'metadata': METADATA,
            'callback_url': CALLBACK_URL
        }
        response = make_mock_response(url=JOB_ID_URL, json_data=data)
        mock_client.session.post.return_value = response

        res = mock_client.submit_job_url(MEDIA_URL, METADATA, CALLBACK_URL)

        assert res == Job(JOB_ID,
                          CREATED_ON,
                          JobStatus.IN_PROGRESS,
                          metadata=METADATA,
                          callback_url=CALLBACK_URL)
        mock_client.session.post.assert_called_once_with(
            JOBS_URL,
            json={
                'media_url': MEDIA_URL,
                'callback_url': CALLBACK_URL,
                'metadata': METADATA
            })

    @pytest.mark.parametrize('url', [None, ''])
    def test_submit_job_url_with_no_media_url(self, url, mock_client):
        with pytest.raises(ValueError, match='media_url must be provided'):
            mock_client.submit_job_url(url)

    @pytest.mark.parametrize('error', get_error_test_cases(
        ['invalid-parameters', 'unauthorized', 'out-of-credit']))
    def test_submit_job_url_with_error_response(self, error, mock_client, make_mock_response):
        status = error.get('status')
        response = make_mock_response(url=JOBS_URL, status=status, json_data=error)
        mock_client.session.post.return_value = response

        with pytest.raises(HTTPError, match=str(status)):
            mock_client.submit_job_url(MEDIA_URL)
        mock_client.session.post.assert_called_once_with(JOBS_URL, json={'media_url': MEDIA_URL})

    def test_submit_job_local_file_with_success(self, mocker, mock_client, make_mock_response):
        created_on = '2018-05-05T23:23:22.29Z'
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': created_on,
            'metadata': METADATA,
            'callback_url': CALLBACK_URL
        }
        response = make_mock_response(url=JOB_ID_URL, json_data=data)
        mock_client.session.post.return_value = response

        with mocker.patch('src.rev_ai.apiclient.open', create=True)() as file:
            res = mock_client.submit_job_local_file(FILENAME, METADATA, CALLBACK_URL)

            assert res == Job(JOB_ID,
                              CREATED_ON,
                              JobStatus.IN_PROGRESS,
                              metadata=METADATA,
                              callback_url=CALLBACK_URL)
            mock_client.session.post.assert_called_once_with(
                JOBS_URL,
                files={
                    'media': (FILENAME, file),
                    'options': (
                        None,
                        json.dumps({
                            'metadata': METADATA,
                            'callback_url': CALLBACK_URL
                        })
                    )
                })

    @pytest.mark.parametrize('filename', [None, ''])
    def test_submit_job_url_with_no_filename(self, filename, mock_client):
        with pytest.raises(ValueError, match='filename must be provided'):
            mock_client.submit_job_local_file(filename, None)

    @pytest.mark.parametrize('error', get_error_test_cases(
        ['invalid-parameters', 'unauthorized', 'out-of-credit']))
    def test_submit_job_local_file_with_error_response(
            self, error, mocker, mock_client, make_mock_response):
        status = error.get('status')
        response = make_mock_response(url=JOBS_URL, status=status, json_data=error)
        mock_client.session.post.return_value = response

        with mocker.patch('src.rev_ai.apiclient.open', create=True)() as file:
            with pytest.raises(HTTPError, match=str(status)):
                mock_client.submit_job_local_file(FILENAME)
            mock_client.session.post.assert_called_once_with(
                JOBS_URL, files={'media': (FILENAME, file), 'options': (None, '{}')})

    def test_delete_job_success(self, mock_client, make_mock_response):
        response = make_mock_response(url=JOB_ID_URL, status=204)
        mock_client.session.delete.return_value = response

        res = mock_client.delete_job(JOB_ID)

        assert res is None
        mock_client.session.delete.assert_called_once_with(JOB_ID_URL)

    @pytest.mark.parametrize('error', get_error_test_cases(
        ['unauthorized', 'job-not-found', 'invalid-job-state']))
    def test_delete_job_with_error_response(
            self, error, mocker, mock_client, make_mock_response):
        status = error.get('status')
        response = make_mock_response(url=JOB_ID_URL, status=status, json_data=error)
        mock_client.session.delete.return_value = response

        with pytest.raises(HTTPError, match=str(status)):
            mock_client.delete_job(JOB_ID)
        mock_client.session.delete.assert_called_once_with(JOB_ID_URL)

    @pytest.mark.parametrize('id', [None, ''])
    def test_delete_job_with_no_id(self, id, mock_client):
        with pytest.raises(ValueError, match='id_ must be provided'):
            mock_client.delete_job(id)



# -*- coding: utf-8 -*-
"""Transcript model"""


class Transcript:
    def __init__(self, monologues):
        """
        :param monologues: list of monologues included in output
        """
        self.monologues = monologues

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return all(a == b for a, b in zip(self.monologues, other.monologues))
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls([Monologue.from_json(monologue) for monologue in json.get('monologues', [])])


class Monologue:
    def __init__(self, speaker, elements):
        """
        :param speaker: speaker identified for this monologue
        :param elements: list of elements spoken in this monologue
        """
        self.speaker = speaker
        self.elements = elements

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return all(a == b for a, b in zip(self.elements, other.elements)) \
                   and self.speaker == other.speaker
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            json['speaker'],
            [Element.from_json(element) for element in json.get('elements', [])])


class Element:
    def __init__(self, type_, value, timestamp, end_timestamp, confidence):
        """
        :param type_: type of element: text, punct, or unknown
        :param value: value of the element
        :param timestamp: time at which this element starts in the audio
        :param end_timestamp: time at which this element ends in the audio
        :param confidence: confidence in this output
        """
        self.type_ = type_
        self.value = value
        self.timestamp = timestamp
        self.end_timestamp = end_timestamp
        self.confidence = confidence

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            json['type'],
            json['value'],
            json.get('ts'),
            json.get('end_ts'),
            json.get('confidence'))
